/*******************************************************************************
* Copyright (C) Maxim Integrated Products, Inc., All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
* OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
*
* Except as contained in this notice, the name of Maxim Integrated
* Products, Inc. shall not be used except as stated in the Maxim Integrated
* Products, Inc. Branding Policy.
*
* The mere transfer of this software does not imply any licenses
* of trade secrets, proprietary technology, copyrights, patents,
* trademarks, maskwork rights, or any other form of intellectual
* property whatsoever. Maxim Integrated Products, Inc. retains all
* ownership rights.
*******************************************************************************/

#include "Button.hpp"

// Border width when button is deselected.
static const int deselectedBorderWidth = 1;

// Border width when button is selected.
static const int selectedBorderWidth = 2;

// Total width of the border region.
static const int borderPadding = selectedBorderWidth + 1;

Button::Button() {
  border.setParent(this);
  border.setBorderWidth(deselectedBorderWidth);
  label.setParent(this);
}

int Button::preferredWidth() const {
  return label.preferredWidth() + 2 * borderPadding;
}

int Button::preferredHeight() const {
  return label.preferredHeight() + 2 * borderPadding;
}

void Button::focusChanged(bool focused) {
  border.setBorderWidth(focused ? selectedBorderWidth : deselectedBorderWidth);
  invalidate(); // Ensure border and text are redrawn together.
}

void Button::resized() {
  border.resize(width(), height());
  label.move(borderPadding, borderPadding);
  label.resize(border.width() - 2 * borderPadding,
               border.height() - 2 * borderPadding);
}

bool Button::doProcessKey(Key key) {
  bool handled = (key == LeftClickKey);
  if (handled && clickedHandler) {
    clickedHandler(this);
  }
  return handled;
}
